/********************************************************************/
/*                                                                  */
/* Program name: EXPIRED                                            */
/*                                                                  */
/* Description: C program that removes expired messages from all    */
/*              queues on a queue manager                           */
/*                                                                  */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Function:                                                        */
/*                                                                  */
/*                                                                  */
/*   EXPIRED is a C program that removes all expired messages from  */
/*   all queues on a queue manager                                  */
/*                                                                  */
/*   If there are no parameters it removes all expired messages     */
/*   from all queues on the default queue manager.                  */
/*   If there is one parameter it uses this as the queue name       */
/*   to remove messages from.                                       */
/*   If there is a second parameter it uses this as the queue       */
/*   manager name.                                                  */
/*                                                                  */
/********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef OS2
  /******************************************************************/
  /* for beep                                                       */
  /******************************************************************/
  #define INCL_DOSPROCESS
  #include <os2.h>
#endif

/********************************************************************/
/* includes for MQI                                                 */
/********************************************************************/
#include <cmqc.h>
#include <cmqcfc.h>
void printfmqftstringlist(MQCFSL* pmqcfsl);
void printreas(MQLONG reason);
void rmvexpired(char * thequeue);

 #define PRINTREAS(param)                                           \
    case param:                                                     \
      printf("Reason = %s\n",#param);                               \
      break;

/********************************************************************/
/* global variable                                                  */
/* the command message                                              */
/********************************************************************/
#ifdef OS2
_Packed
#endif
struct cmdmsg2 {
                 MQCFH  pcfheader;
                 MQCFST qname;
                 MQCFIN qtype;
               } cmdmsg = {MQCFH_DEFAULT,
                       MQCFST_DEFAULT,
                       MQCFIN_DEFAULT};
MQCFH    *outmsg;                    /* evtmsg message buffer       */
MQHCONN  Hcon;                       /* connection handle           */


int main(int argc, char **argv)
{
  /******************************************************************/
  /* Declare MQI structures needed                                  */
  /******************************************************************/
  MQOD     od = {MQOD_DEFAULT};      /* Object Descriptor           */
  MQOD     replod = {MQOD_DEFAULT};  /* Object Descriptor           */
  MQMD     md = {MQMD_DEFAULT};      /* Message Descriptor          */
  MQGMO    gmo = {MQGMO_DEFAULT};    /* get message options         */
  MQPMO    pmo = {MQPMO_DEFAULT};    /* get message options         */
  MQMD     PMsgDescriptor= { MQMD_DEFAULT };

  /******************************************************************/
  /* note, uses defaults where it can                               */
  /******************************************************************/
  MQHOBJ   Hobj;                     /* object handle               */
  MQHOBJ   replHobj;                 /* object handle               */
  MQHOBJ   QueueHandle;              /* object handle               */
  MQLONG   O_options;                /* MQOPEN options              */
  MQLONG   C_options;                /* MQCLOSE options             */
  MQLONG   CompCode;                 /* completion code             */
  MQLONG   OpenCode;                 /* MQOPEN completion code      */
  MQLONG   replOpenCode;             /* MQOPEN completion code      */
  MQLONG   Reason;                   /* reason code                 */
  MQLONG   CReason;                  /* reason code for MQCONN      */
  MQLONG   buflen;                   /* buffer length               */
  MQLONG   cmdmsglen;                /* message length received     */
  MQLONG   outmsglen;                /* message length received     */
  MQCHAR   command[1100];            /* call command string ...     */
  MQCHAR   p1[600];                  /* ApplId insert               */
  MQCHAR   p2[900];                  /* cmdmsg insert               */
  MQCHAR   p3[600];                  /* Environment insert          */
  MQLONG   mytype;                   /* saved application type      */
  char     QMName[50];               /* queue manager name          */
  char     servstr[100];
  MQCFST  *paras;                    /* the parameters              */
  int      counter;                  /* loop counter                */
  int      flret;                    /* used in flush               */

   /******************************************************************/
   /* See whether they want help                                     */
   /******************************************************************/
   if (argc > 1 && argv[1][0] == '?')
   {
     printf("Usage: expired [Queue] [QMgrName]\n" );
     exit(0);
   }

  /******************************************************************/
  /* Connect to queue manager                                       */
  /******************************************************************/
  QMName[0] = 0;                      /* default queue manager      */
  if (argc > 2)
    strncpy(QMName, argv[2],sizeof(MQCHAR48));
  MQCONN(QMName,                      /* queue manager              */
         &Hcon,                       /* connection handle          */
         &CompCode,                   /* completion code            */
         &CReason);                   /* reason code                */

  /******************************************************************/
  /* report reason and stop if it failed                            */
  /******************************************************************/
  if (CompCode == MQCC_FAILED)
  {
    printf("MQCONN ended with reason code %ld\n", CReason);
    printreas(CReason);
    exit(CReason);
  }

  if (argc > 1)
  {
    rmvexpired(argv[1]);
  }
  else
  {

    /******************************************************************/
    /* Initialize object descriptor for subject queue                 */
    /* Open the reply queue. We have to do this first to get it's     */
    /* name                                                           */
    /******************************************************************/
    strcpy(replod.ObjectName, "SYSTEM.DEFAULT.MODEL.QUEUE");

    /******************************************************************/
    /* Open the replyto queue for input; exclusive or shared.  Use of */
    /* the queue is controlled by the queue definition here           */
    /******************************************************************/
    MQOPEN(Hcon,                        /* connection handle          */
           &replod,                     /* object descriptor for queue*/
           MQOO_INPUT_AS_Q_DEF          /* open queue for input       */
          + MQOO_FAIL_IF_QUIESCING      /* but not if MQM stopping    */
          + MQOO_BROWSE,                /* open options               */
           &replHobj,                   /* object handle              */
           &CompCode,                   /* completion code            */
           &Reason);                    /* reason code                */

    /******************************************************************/
    /* report reason, if any; stop if failed                          */
    /******************************************************************/
    if (Reason != MQRC_NONE)
    {
      printf("MQOPEN (%s) ==> %ld\n", replod.ObjectName, Reason);
      printreas(Reason);
    }

    replOpenCode = CompCode;            /* keep for conditional close */

    /******************************************************************/
    /* run the command server, it shouldn't matter if it's already    */
    /* running                                                        */
    /* There are ways of finding out the default queue mgr name but   */
    /* it's too much hassle                                           */
    /******************************************************************/
    if (strlen(QMName) != 0)
    {
      sprintf(servstr,"strmqcsv %s",QMName);
      system(servstr);
    }
    else
    {
      printf("If this hangs it's because you haven't"
              " started the command server\n");
    }

    /******************************************************************/
    /*                                                                */
    /* First we need a list of queues on the queue manager. To get    */
    /* this we need to put a request to the command server            */
    /*                                                                */
    /******************************************************************/

    /******************************************************************/
    /* Initialize object descriptor for subject queue                 */
    /******************************************************************/
    strcpy(od.ObjectName, "SYSTEM.ADMIN.COMMAND.QUEUE");

    /******************************************************************/
    /* Open the reply queue for input; exclusive or shared.  Use of   */
    /* the queue is controlled by the queue definition here           */
    /******************************************************************/
    O_options = MQOO_OUTPUT             /* open queue for putting     */
          + MQOO_FAIL_IF_QUIESCING;     /* but not if MQM stopping    */
    MQOPEN(Hcon,                        /* connection handle          */
           &od,                         /* object descriptor for queue*/
           O_options,                   /* open options               */
           &QueueHandle,                /* object handle              */
           &CompCode,                   /* completion code            */
           &Reason);                    /* reason code                */

    /******************************************************************/
    /* report reason, if any; stop if failed                          */
    /******************************************************************/
    if (Reason != MQRC_NONE)
    {
      printf("MQOPEN (%s) ==> %ld\n", od.ObjectName, Reason);
      printreas(Reason);
    }

    if (CompCode == MQCC_FAILED)
    {
      /****************************************************************/
      /* get out                                                      */
      /****************************************************************/
      printf( "DO MQDISC\n" );

      MQDISC(&Hcon,
             &CompCode,
             &Reason );

      printf("MQDISC       Reason code    = %d\n", Reason);
      printf("        Hex  Reason code    = %x\n", Reason);
      printf("             Completion code= %d\n", CompCode);
      puts("***\n");
      flret = fflush(stdout);
      puts("Bye...\n\n");
      return 1;
    }

    /******************************************************************/
    /* set the message descriptor to say it's a command message       */
    /******************************************************************/
    memcpy(PMsgDescriptor.Format,
           MQFMT_ADMIN,
           sizeof(PMsgDescriptor.Format));

    memcpy(PMsgDescriptor.ReplyToQ,
           replod.ObjectName,
           sizeof(PMsgDescriptor.ReplyToQ));

    PMsgDescriptor.MsgType = MQMT_REQUEST;

    cmdmsg.pcfheader.Command = MQCMD_INQUIRE_Q_NAMES;
    cmdmsg.pcfheader.ParameterCount = 2;

    /*******************************************************************/
    /* set the parameter to say that we want to know about all queues  */
    /* of the type specified in the other parameter                    */
    /*******************************************************************/
    cmdmsg.qname.Parameter    = MQCA_Q_NAME;
    cmdmsg.qname.String[0]    = '*';
    cmdmsg.qname.StringLength = 1;

    /*******************************************************************/
    /* note that although the length is actually only increased by 1   */
    /* the docs say it should be a multiple of 4                       */
    /*******************************************************************/
    cmdmsg.qname.StrucLength = MQCFST_STRUC_LENGTH_FIXED + 4;

    /*******************************************************************/
    /* set the parameter to say that we want to know about remote      */
    /* queues                                                          */
    /*******************************************************************/
    cmdmsg.qtype.Parameter    = MQIA_Q_TYPE;
    cmdmsg.qtype.Value        = MQQT_LOCAL;

    /******************************************************************/
    /* put the command message                                        */
    /******************************************************************/
    MQPUT(Hcon,
          QueueHandle,
          &PMsgDescriptor,
          &pmo,
          (long int)sizeof(cmdmsg),
          &cmdmsg,
          &CompCode,
          &Reason );

    /*******************************************************************/
    /* if put failed then close, disconnect and exit                   */
    /*******************************************************************/
    if (CompCode)
    {
      printf("MQPUT      Reason code    = %d\n", Reason);
      printf("      Hex  Reason code    = %x\n", Reason);
      printreas(Reason);
      printf("           Completion code= %d\n", CompCode);
      puts("***\n");
      flret = fflush(stdout);

      printf( "DO MQCLOSE -MQPUT Failed\n" );

      MQCLOSE(Hcon,
              &QueueHandle,
              0,
              &CompCode,
              &Reason);

      printf("MQCLOSE      Reason code    = %d\n", Reason);
      printf("        Hex  Reason code    = %x\n", Reason);
      printreas(Reason);
      printf("             Completion code= %d\n", CompCode);
      puts("***\n");
      flret = fflush(stdout);

      printf( "DO MQDISC\n" );

      MQDISC(&Hcon,
             &CompCode,
             &Reason );

      printf("MQDISC       Reason code    = %d\n", Reason);
      printf("        Hex  Reason code    = %x\n", Reason);
      printreas(Reason);
      printf("             Completion code= %d\n", CompCode);
      puts("***\n");
      flret = fflush(stdout);
      puts("Bye...\n\n");
      return 1;
    }

    /******************************************************************/
    /* close the putmsg queue                                         */
    /******************************************************************/
    MQCLOSE(Hcon,                       /* connection handle          */
            &QueueHandle,               /* object handle              */
            0,
            &CompCode,                  /* completion code            */
            &Reason);                   /* reason code                */

    /******************************************************************/
    /* report reason, if any                                          */
    /******************************************************************/
    if (Reason != MQRC_NONE)
    {
      printf("MQCLOSE ==> %ld\n", Reason);
      printreas(Reason);
    }

    /******************************************************************/
    /*   Get messages from the message queue                          */
    /******************************************************************/
    while (CompCode != MQCC_FAILED)
    {
      /****************************************************************/
      /* I don't know how big this message is so just get the         */
      /* descriptor first                                             */
      /****************************************************************/
      gmo.Options = MQGMO_WAIT + MQGMO_LOCK
         + MQGMO_BROWSE_FIRST + MQGMO_ACCEPT_TRUNCATED_MSG;
                                        /* wait for new messages      */
      gmo.WaitInterval = MQWI_UNLIMITED;/* no time limit              */
      buflen = 0;                       /* ammount of message to get  */

      /****************************************************************/
      /* clear selectors to get messages in sequence                  */
      /****************************************************************/
      memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
      memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));

      /****************************************************************/
      /* wait for reply message                                       */
      /****************************************************************/
      MQGET(Hcon,                       /* connection handle          */
            replHobj,                   /* object handle              */
            &md,                        /* message descriptor         */
            &gmo,                       /* get message options        */
            buflen,                     /* buffer length              */
            outmsg,                     /* outmsg message buffer      */
            &outmsglen,                 /* message length             */
            &CompCode,                  /* completion code            */
            &Reason);                   /* reason code                */

      /****************************************************************/
      /* report reason, if any                                        */
      /****************************************************************/
      if (Reason != MQRC_NONE && Reason != MQRC_TRUNCATED_MSG_ACCEPTED)
      {
        printf("MQGET ==> %ld\n", Reason);
        printreas(Reason);
      }
      else
      {
        gmo.Options = MQGMO_NO_WAIT + MQGMO_MSG_UNDER_CURSOR;
        buflen = outmsglen;             /* ammount of message to get  */
        outmsg = malloc(buflen);
        if (outmsg != NULL)
        {
          /************************************************************/
          /* clear selectors to get messages in sequence              */
          /************************************************************/
          memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
          memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));

          /************************************************************/
          /* get the reply message                                    */
          /************************************************************/
          MQGET(Hcon,                     /* connection handle        */
                replHobj,                 /* object handle            */
                &md,                      /* message descriptor       */
                &gmo,                     /* get message options      */
                buflen,                   /* buffer length            */
                outmsg,                   /* outmsg message buffer    */
                &outmsglen,               /* message length           */
                &CompCode,                /* completion code          */
                &Reason);                 /* reason code              */

          /************************************************************/
          /* report reason, if any                                    */
          /************************************************************/
          if (Reason != MQRC_NONE)
          {
            printf("MQGET ==> %ld\n", Reason);
            printreas(Reason);
          }
        }
        else
        {
          CompCode = MQCC_FAILED;
        }
      }
      /****************************************************************/
      /* . . . process each message received                          */
      /****************************************************************/
      if (CompCode != MQCC_FAILED)
      {
        if (outmsglen != buflen)
          printf("DataLength = %ld?\n", outmsglen);
        else
        {
          /************************************************************/
          /* right let's look at the data                             */
          /************************************************************/
          if (outmsg->Type != MQCFT_RESPONSE)
          {
            printf("something's wrong this isn't a response message"
                   " it's type is %ld\n",outmsg->Type);
          }
          else
          {
            printf("reply message arrived\n");
            if (outmsg->StrucLength != MQCFH_STRUC_LENGTH)
            {
              printf("it's the wrong length, %ld\n",outmsg->StrucLength);
            }
            else
            {
              printf("it's the right length\n");
              if (outmsg->Version != MQCFH_VERSION_1)
              {
                printf("it's the wrong version, %ld\n",outmsg->Version);
              }
              else
              {
                printf("it's the right version\n");
                if (outmsg->Command != MQCMD_INQUIRE_Q_NAMES)
                {
                  printf("it's the wrong sort of response message, %ld\n",
                          outmsg->Command);
                }
                else
                {
                  printf("it's the right kind of response message\n");
                  printf("it's sequence number is %ld\n",
                            outmsg->MsgSeqNumber);

                  switch (outmsg->Control)
                  {
                    case MQCFC_LAST:
                      printf("it's control option is MQCFC_LAST\n");
                      /*************************************************/
                      /* to end loop                                   */
                      /*************************************************/
                      CompCode = MQCC_FAILED;
                      break;
                    case MQCFC_NOT_LAST:
                      printf("it's control option is MQCFC_NOT_LAST\n");
                      break;
                    default:
                      printf("it's control option is wrong, %ld\n",
                             outmsg->Control);
                      break;
                  }

                    {
                      if (outmsg->CompCode == MQCC_OK)
                      {
                        printf("CompCode is OK\n");
                      }
                      else
                      {
                        if (outmsg->CompCode == MQCC_WARNING)
                        {
                          printf("CompCode is WARNING\n");
                        }
                        else
                        {
                          if (outmsg->CompCode == MQCC_FAILED)
                          {
                            printf("CompCode is FAILED\n");
                          }
                          else
                          {
                            printf("CompCode is wrong, %ld\n",
                                    outmsg->CompCode);
                          }
                        }
                      }
                      printf("parameter count is %ld\n",
                                    outmsg->ParameterCount);
                      printreas(outmsg->Reason);
                      /************************************************/
                      /* get a pointer to the start of the parameters */
                      /************************************************/
                      paras = (MQCFST *)(outmsg + 1);
                      counter = 1;
                      while (counter <= outmsg->ParameterCount)
                      {
                        switch (paras->Type)
                        {
                          case MQCFT_STRING_LIST:
                            printfmqftstringlist((MQCFSL*)paras);
                            paras = (MQCFST *)((char *)paras
                                                 + paras->StrucLength);
                            break;
                          default:
                            printf("unexpected parameter type, %ld\n",
                                   paras->Type);
                            counter = outmsg->ParameterCount;
                            break;
                        }
                        counter++;
                      }
                    }
                }
              }
            }
          }
        }   /* end outmsg action             */
        free(outmsg);
      }     /* end process for successful GET */
    }       /* end message processing loop    */

    /******************************************************************/
    /* close the reply queue - if it was opened                       */
    /******************************************************************/
    if (replOpenCode != MQCC_FAILED)
    {
      C_options = 0;                  /* no close options             */
      MQCLOSE(Hcon,                   /* connection handle            */
             &replHobj,               /* object handle                */
             C_options,
             &CompCode,               /* completion code              */
             &Reason);                /* reason code                  */
      /****************************************************************/
      /* report reason, if any                                        */
      /****************************************************************/
      if (Reason != MQRC_NONE)
      {
        printf("MQCLOSE ==> %ld\n", Reason);
        printreas(Reason);
      }
    }
  }

  /******************************************************************/
  /* Disconnect from MQM  (unless previously connected)             */
  /******************************************************************/
  if (CReason != MQRC_ALREADY_CONNECTED)
  {
    MQDISC(&Hcon,                   /* connection handle            */
           &CompCode,               /* completion code              */
           &Reason);                /* reason code                  */
    /****************************************************************/
    /* report reason, if any                                        */
    /****************************************************************/
    if (Reason != MQRC_NONE)
    {
      printf("MQDISC ended with reason code %ld\n", Reason);
      printreas(Reason);
    }
  }
}
/********************************************************************/
/*                                                                  */
/* END OF EXPIRED                                                   */
/*                                                                  */
/********************************************************************/

#define PRINTTHEPARAM(param)                                       \
   case param:                                                     \
     printf("%s\n",#param);                                        \
     break;

void printfmqftstringlist(MQCFSL* pmqcfsl)
{
  MQLONG numqueues;
  char * thequeue;

  switch (pmqcfsl->Parameter)
  {
    PRINTTHEPARAM(MQCA_APPL_ID)
    PRINTTHEPARAM(MQCA_BACKOUT_REQ_Q_NAME)
    PRINTTHEPARAM(MQCA_BASE_Q_NAME)
    PRINTTHEPARAM(MQCA_COMMAND_INPUT_Q_NAME)
    PRINTTHEPARAM(MQCA_CREATION_DATE)
    PRINTTHEPARAM(MQCA_CREATION_TIME)
    PRINTTHEPARAM(MQCA_DEAD_LETTER_Q_NAME)
    PRINTTHEPARAM(MQCA_DEF_XMIT_Q_NAME)
    PRINTTHEPARAM(MQCA_ENV_DATA)
    PRINTTHEPARAM(MQCA_INITIATION_Q_NAME)
    PRINTTHEPARAM(MQCA_NAMES)
    PRINTTHEPARAM(MQCA_PROCESS_DESC)
    PRINTTHEPARAM(MQCA_PROCESS_NAME)
    PRINTTHEPARAM(MQCA_Q_DESC)
    PRINTTHEPARAM(MQCA_Q_MGR_DESC)
    PRINTTHEPARAM(MQCA_Q_MGR_NAME)
    PRINTTHEPARAM(MQCA_Q_NAME)
    PRINTTHEPARAM(MQCA_REMOTE_Q_MGR_NAME)
    PRINTTHEPARAM(MQCA_REMOTE_Q_NAME)
    PRINTTHEPARAM(MQCA_STORAGE_CLASS)
    PRINTTHEPARAM(MQCA_TRIGGER_DATA)
    PRINTTHEPARAM(MQCA_USER_DATA)
    PRINTTHEPARAM(MQCA_XMIT_Q_NAME)
    PRINTTHEPARAM(MQCACF_FROM_Q_NAME)
    PRINTTHEPARAM(MQCACF_TO_Q_NAME)
    PRINTTHEPARAM(MQCACF_FROM_PROCESS_NAME)
    PRINTTHEPARAM(MQCACF_TO_PROCESS_NAME)
    PRINTTHEPARAM(MQCACF_FROM_CHANNEL_NAME)
    PRINTTHEPARAM(MQCACF_TO_CHANNEL_NAME)
    PRINTTHEPARAM(MQCACF_Q_NAMES)
    PRINTTHEPARAM(MQCACF_PROCESS_NAMES)
    PRINTTHEPARAM(MQCACF_ESCAPE_TEXT)
    PRINTTHEPARAM(MQCACF_LOCAL_Q_NAMES)
    PRINTTHEPARAM(MQCACF_MODEL_Q_NAMES)
    PRINTTHEPARAM(MQCACF_ALIAS_Q_NAMES)
    PRINTTHEPARAM(MQCACF_REMOTE_Q_NAMES)
    PRINTTHEPARAM(MQCACF_SENDER_CHANNEL_NAMES)
    PRINTTHEPARAM(MQCACF_SERVER_CHANNEL_NAMES)
    PRINTTHEPARAM(MQCACF_REQUESTER_CHANNEL_NAMES)
    PRINTTHEPARAM(MQCACF_RECEIVER_CHANNEL_NAMES)
    PRINTTHEPARAM(MQCACF_OBJECT_Q_MGR_NAME)
    PRINTTHEPARAM(MQCACF_APPL_NAME)
    PRINTTHEPARAM(MQCACF_USER_IDENTIFIER)
    PRINTTHEPARAM(MQCACF_AUX_ERROR_DATA_STR_1)
    PRINTTHEPARAM(MQCACF_AUX_ERROR_DATA_STR_2)
    PRINTTHEPARAM(MQCACF_AUX_ERROR_DATA_STR_3)
    PRINTTHEPARAM(MQCACH_CHANNEL_NAME)
    PRINTTHEPARAM(MQCACH_DESC)
    PRINTTHEPARAM(MQCACH_MODE_NAME)
    PRINTTHEPARAM(MQCACH_TP_NAME)
    PRINTTHEPARAM(MQCACH_XMIT_Q_NAME)
    PRINTTHEPARAM(MQCACH_CONNECTION_NAME)
    PRINTTHEPARAM(MQCACH_MCA_NAME)
    PRINTTHEPARAM(MQCACH_SEC_EXIT_NAME)
    PRINTTHEPARAM(MQCACH_MSG_EXIT_NAME)
    PRINTTHEPARAM(MQCACH_SEND_EXIT_NAME)
    PRINTTHEPARAM(MQCACH_RCV_EXIT_NAME)
    PRINTTHEPARAM(MQCACH_CHANNEL_NAMES)
    PRINTTHEPARAM(MQCACH_SEC_EXIT_USER_DATA)
    PRINTTHEPARAM(MQCACH_MSG_EXIT_USER_DATA)
    PRINTTHEPARAM(MQCACH_SEND_EXIT_USER_DATA)
    PRINTTHEPARAM(MQCACH_RCV_EXIT_USER_DATA)
    PRINTTHEPARAM(MQCACH_USER_ID)
    PRINTTHEPARAM(MQCACH_PASSWORD)
    PRINTTHEPARAM(MQCACH_LAST_MSG_TIME)
    PRINTTHEPARAM(MQCACH_LAST_MSG_DATE)
    PRINTTHEPARAM(MQCACH_MCA_USER_ID)
    PRINTTHEPARAM(MQCACH_CHANNEL_START_TIME)
    PRINTTHEPARAM(MQCACH_CHANNEL_START_DATE)
    PRINTTHEPARAM(MQCACH_MCA_JOB_NAME)
    PRINTTHEPARAM(MQCACH_LAST_LUWID)
    PRINTTHEPARAM(MQCACH_CURRENT_LUWID)
    PRINTTHEPARAM(MQCACH_FORMAT_NAME)
    default:
      printf("Invalid parameter, %ld\n",pmqcfsl->Parameter);
      break;
  }
  printf("There are %ld strings\n",pmqcfsl->Count);
  if (pmqcfsl->Count != 0)
  {
    thequeue = malloc(pmqcfsl->StringLength + 1);
    if (thequeue != NULL)
    {
      for (numqueues = 0; numqueues < pmqcfsl->Count; numqueues++)
      {
        strncpy(thequeue,
                pmqcfsl->Strings + (numqueues * pmqcfsl->StringLength),
                pmqcfsl->StringLength);
        printf("%s\n",thequeue);
        rmvexpired(thequeue);
      }
    }
  }
}


/*********************************************************************/
/* Print the reason code                                             */
/*********************************************************************/
void printreas(MQLONG reason)
{
  switch (reason)
  {
    PRINTREAS(MQRCCF_CFH_TYPE_ERROR)
    PRINTREAS(MQRCCF_CFH_LENGTH_ERROR)
    PRINTREAS(MQRCCF_CFH_VERSION_ERROR)
    PRINTREAS(MQRCCF_CFH_MSG_SEQ_NUMBER_ERR)
    PRINTREAS(MQRCCF_CFH_CONTROL_ERROR)
    PRINTREAS(MQRCCF_CFH_PARM_COUNT_ERROR)
    PRINTREAS(MQRCCF_CFH_COMMAND_ERROR)
    PRINTREAS(MQRCCF_COMMAND_FAILED)
    PRINTREAS(MQRCCF_CFIN_LENGTH_ERROR)
    PRINTREAS(MQRCCF_CFST_LENGTH_ERROR)
    PRINTREAS(MQRCCF_CFST_STRING_LENGTH_ERR)
    PRINTREAS(MQRCCF_FORCE_VALUE_ERROR)
    PRINTREAS(MQRCCF_STRUCTURE_TYPE_ERROR)
    PRINTREAS(MQRCCF_CFIN_PARM_ID_ERROR)
    PRINTREAS(MQRCCF_CFST_PARM_ID_ERROR)
    PRINTREAS(MQRCCF_MSG_LENGTH_ERROR)
    PRINTREAS(MQRCCF_CFIN_DUPLICATE_PARM)
    PRINTREAS(MQRCCF_CFST_DUPLICATE_PARM)
    PRINTREAS(MQRCCF_PARM_COUNT_TOO_SMALL)
    PRINTREAS(MQRCCF_PARM_COUNT_TOO_BIG)
    PRINTREAS(MQRCCF_Q_ALREADY_IN_CELL)
    PRINTREAS(MQRCCF_Q_TYPE_ERROR)
    PRINTREAS(MQRCCF_MD_FORMAT_ERROR)
    PRINTREAS(MQRCCF_REPLACE_VALUE_ERROR)
    PRINTREAS(MQRCCF_CFIL_DUPLICATE_VALUE)
    PRINTREAS(MQRCCF_CFIL_COUNT_ERROR)
    PRINTREAS(MQRCCF_CFIL_LENGTH_ERROR)
    PRINTREAS(MQRCCF_QUIESCE_VALUE_ERROR)
    PRINTREAS(MQRCCF_MSG_SEQ_NUMBER_ERROR)
    PRINTREAS(MQRCCF_PING_DATA_COUNT_ERROR)
    PRINTREAS(MQRCCF_PING_DATA_COMPARE_ERROR)
    PRINTREAS(MQRCCF_CHANNEL_TYPE_ERROR)
    PRINTREAS(MQRCCF_PARM_SEQUENCE_ERROR)
    PRINTREAS(MQRCCF_XMIT_PROTOCOL_TYPE_ERR)
    PRINTREAS(MQRCCF_BATCH_SIZE_ERROR)
    PRINTREAS(MQRCCF_DISC_INT_ERROR)
    PRINTREAS(MQRCCF_SHORT_RETRY_ERROR)
    PRINTREAS(MQRCCF_SHORT_TIMER_ERROR)
    PRINTREAS(MQRCCF_LONG_RETRY_ERROR)
    PRINTREAS(MQRCCF_LONG_TIMER_ERROR)
    PRINTREAS(MQRCCF_SEQ_NUMBER_WRAP_ERROR)
    PRINTREAS(MQRCCF_MAX_MSG_LENGTH_ERROR)
    PRINTREAS(MQRCCF_PUT_AUTH_ERROR)
    PRINTREAS(MQRCCF_PURGE_VALUE_ERROR)
    PRINTREAS(MQRCCF_CFIL_PARM_ID_ERROR)
    PRINTREAS(MQRCCF_MSG_TRUNCATED)
    PRINTREAS(MQRCCF_CCSID_ERROR)
    PRINTREAS(MQRCCF_ENCODING_ERROR)
    PRINTREAS(MQRCCF_DATA_CONV_VALUE_ERROR)
    PRINTREAS(MQRCCF_INDOUBT_VALUE_ERROR)
    PRINTREAS(MQRCCF_ESCAPE_TYPE_ERROR)
    PRINTREAS(MQRCCF_CHANNEL_TABLE_ERROR)
    PRINTREAS(MQRCCF_MCA_TYPE_ERROR)
    PRINTREAS(MQRCCF_CHL_INST_TYPE_ERROR)
    PRINTREAS(MQRCCF_CHL_STATUS_NOT_FOUND)
    PRINTREAS(MQRCCF_OBJECT_ALREADY_EXISTS)
    PRINTREAS(MQRCCF_OBJECT_WRONG_TYPE)
    PRINTREAS(MQRCCF_LIKE_OBJECT_WRONG_TYPE)
    PRINTREAS(MQRCCF_OBJECT_OPEN)
    PRINTREAS(MQRCCF_ATTR_VALUE_ERROR)
    PRINTREAS(MQRCCF_UNKNOWN_Q_MGR)
    PRINTREAS(MQRCCF_Q_WRONG_TYPE)
    PRINTREAS(MQRCCF_OBJECT_NAME_ERROR)
    PRINTREAS(MQRCCF_ALLOCATE_FAILED)
    PRINTREAS(MQRCCF_HOST_NOT_AVAILABLE)
    PRINTREAS(MQRCCF_CONFIGURATION_ERROR)
    PRINTREAS(MQRCCF_CONNECTION_REFUSED)
    PRINTREAS(MQRCCF_ENTRY_ERROR)
    PRINTREAS(MQRCCF_SEND_FAILED)
    PRINTREAS(MQRCCF_RECEIVED_DATA_ERROR)
    PRINTREAS(MQRCCF_RECEIVE_FAILED)
    PRINTREAS(MQRCCF_CONNECTION_CLOSED)
    PRINTREAS(MQRCCF_NO_STORAGE)
    PRINTREAS(MQRCCF_NO_COMMS_MANAGER)
    PRINTREAS(MQRCCF_LISTENER_NOT_STARTED)
    PRINTREAS(MQRCCF_BIND_FAILED)
    PRINTREAS(MQRCCF_CHANNEL_INDOUBT)
    PRINTREAS(MQRCCF_MQCONN_FAILED)
    PRINTREAS(MQRCCF_MQOPEN_FAILED)
    PRINTREAS(MQRCCF_MQGET_FAILED)
    PRINTREAS(MQRCCF_MQPUT_FAILED)
    PRINTREAS(MQRCCF_PING_ERROR)
    PRINTREAS(MQRCCF_CHANNEL_IN_USE)
    PRINTREAS(MQRCCF_CHANNEL_NOT_FOUND)
    PRINTREAS(MQRCCF_UNKNOWN_REMOTE_CHANNEL)
    PRINTREAS(MQRCCF_REMOTE_QM_UNAVAILABLE)
    PRINTREAS(MQRCCF_REMOTE_QM_TERMINATING)
    PRINTREAS(MQRCCF_MQINQ_FAILED)
    PRINTREAS(MQRCCF_NOT_XMIT_Q)
    PRINTREAS(MQRCCF_CHANNEL_DISABLED)
    PRINTREAS(MQRCCF_USER_EXIT_NOT_AVAILABLE)
    PRINTREAS(MQRCCF_COMMIT_FAILED)
    PRINTREAS(MQRCCF_CHANNEL_ALREADY_EXISTS)
    PRINTREAS(MQRCCF_DATA_TOO_LARGE)
    PRINTREAS(MQRCCF_CHANNEL_NAME_ERROR)
    PRINTREAS(MQRCCF_XMIT_Q_NAME_ERROR)
    PRINTREAS(MQRCCF_MCA_NAME_ERROR)
    PRINTREAS(MQRCCF_SEND_EXIT_NAME_ERROR)
    PRINTREAS(MQRCCF_SEC_EXIT_NAME_ERROR)
    PRINTREAS(MQRCCF_MSG_EXIT_NAME_ERROR)
    PRINTREAS(MQRCCF_RCV_EXIT_NAME_ERROR)
    PRINTREAS(MQRCCF_XMIT_Q_NAME_WRONG_TYPE)
    PRINTREAS(MQRCCF_MCA_NAME_WRONG_TYPE)
    PRINTREAS(MQRCCF_DISC_INT_WRONG_TYPE)
    PRINTREAS(MQRCCF_SHORT_RETRY_WRONG_TYPE)
    PRINTREAS(MQRCCF_SHORT_TIMER_WRONG_TYPE)
    PRINTREAS(MQRCCF_LONG_RETRY_WRONG_TYPE)
    PRINTREAS(MQRCCF_LONG_TIMER_WRONG_TYPE)
    PRINTREAS(MQRCCF_PUT_AUTH_WRONG_TYPE)
    PRINTREAS(MQRCCF_MISSING_CONN_NAME)
    PRINTREAS(MQRCCF_CONN_NAME_ERROR)
    PRINTREAS(MQRCCF_MQSET_FAILED)
    PRINTREAS(MQRCCF_CHANNEL_NOT_ACTIVE)
    PRINTREAS(MQRCCF_TERMINATED_BY_SEC_EXIT)
    PRINTREAS(MQRCCF_DYNAMIC_Q_SCOPE_ERROR)
    PRINTREAS(MQRCCF_CELL_DIR_NOT_AVAILABLE)
    PRINTREAS(MQRC_NONE)
    PRINTREAS(MQRC_ALIAS_BASE_Q_TYPE_ERROR)
    PRINTREAS(MQRC_ALREADY_CONNECTED)
    PRINTREAS(MQRC_BACKED_OUT)
    PRINTREAS(MQRC_BUFFER_ERROR)
    PRINTREAS(MQRC_BUFFER_LENGTH_ERROR)
    PRINTREAS(MQRC_CHAR_ATTR_LENGTH_ERROR)
    PRINTREAS(MQRC_CHAR_ATTRS_ERROR)
    PRINTREAS(MQRC_CHAR_ATTRS_TOO_SHORT)
    PRINTREAS(MQRC_CONNECTION_BROKEN)
    PRINTREAS(MQRC_DATA_LENGTH_ERROR)
    PRINTREAS(MQRC_DYNAMIC_Q_NAME_ERROR)
    PRINTREAS(MQRC_ENVIRONMENT_ERROR)
    PRINTREAS(MQRC_EXPIRY_ERROR)
    PRINTREAS(MQRC_FEEDBACK_ERROR)
    PRINTREAS(MQRC_GET_INHIBITED)
    PRINTREAS(MQRC_HANDLE_NOT_AVAILABLE)
    PRINTREAS(MQRC_HCONN_ERROR)
    PRINTREAS(MQRC_HOBJ_ERROR)
    PRINTREAS(MQRC_INHIBIT_VALUE_ERROR)
    PRINTREAS(MQRC_INT_ATTR_COUNT_ERROR)
    PRINTREAS(MQRC_INT_ATTR_COUNT_TOO_SMALL)
    PRINTREAS(MQRC_INT_ATTRS_ARRAY_ERROR)
    PRINTREAS(MQRC_SYNCPOINT_LIMIT_REACHED)
    PRINTREAS(MQRC_MAX_CONNS_LIMIT_REACHED)
    PRINTREAS(MQRC_MD_ERROR)
    PRINTREAS(MQRC_MISSING_REPLY_TO_Q)
    PRINTREAS(MQRC_MSG_TYPE_ERROR)
    PRINTREAS(MQRC_MSG_TOO_BIG_FOR_Q)
    PRINTREAS(MQRC_MSG_TOO_BIG_FOR_Q_MGR)
    PRINTREAS(MQRC_NO_MSG_AVAILABLE)
    PRINTREAS(MQRC_NO_MSG_UNDER_CURSOR)
    PRINTREAS(MQRC_NOT_AUTHORIZED)
    PRINTREAS(MQRC_NOT_OPEN_FOR_BROWSE)
    PRINTREAS(MQRC_NOT_OPEN_FOR_INPUT)
    PRINTREAS(MQRC_NOT_OPEN_FOR_INQUIRE)
    PRINTREAS(MQRC_NOT_OPEN_FOR_OUTPUT)
    PRINTREAS(MQRC_NOT_OPEN_FOR_SET)
    PRINTREAS(MQRC_OBJECT_CHANGED)
    PRINTREAS(MQRC_OBJECT_IN_USE)
    PRINTREAS(MQRC_OBJECT_TYPE_ERROR)
    PRINTREAS(MQRC_OD_ERROR)
    PRINTREAS(MQRC_OPTION_NOT_VALID_FOR_TYPE)
    PRINTREAS(MQRC_OPTIONS_ERROR)
    PRINTREAS(MQRC_PERSISTENCE_ERROR)
    PRINTREAS(MQRC_PERSISTENT_NOT_ALLOWED)
    PRINTREAS(MQRC_PRIORITY_EXCEEDS_MAXIMUM)
    PRINTREAS(MQRC_PRIORITY_ERROR)
    PRINTREAS(MQRC_PUT_INHIBITED)
    PRINTREAS(MQRC_Q_DELETED)
    PRINTREAS(MQRC_Q_FULL)
    PRINTREAS(MQRC_Q_NOT_EMPTY)
    PRINTREAS(MQRC_Q_SPACE_NOT_AVAILABLE)
    PRINTREAS(MQRC_Q_TYPE_ERROR)
    PRINTREAS(MQRC_Q_MGR_NAME_ERROR)
    PRINTREAS(MQRC_Q_MGR_NOT_AVAILABLE)
    PRINTREAS(MQRC_REPORT_OPTIONS_ERROR)
    PRINTREAS(MQRC_SECOND_MARK_NOT_ALLOWED)
    PRINTREAS(MQRC_SECURITY_ERROR)
    PRINTREAS(MQRC_SELECTOR_COUNT_ERROR)
    PRINTREAS(MQRC_SELECTOR_LIMIT_EXCEEDED)
    PRINTREAS(MQRC_SELECTOR_ERROR)
    PRINTREAS(MQRC_SELECTOR_NOT_FOR_TYPE)
    PRINTREAS(MQRC_SIGNAL_OUTSTANDING)
    PRINTREAS(MQRC_SIGNAL_REQUEST_ACCEPTED)
    PRINTREAS(MQRC_STORAGE_NOT_AVAILABLE)
    PRINTREAS(MQRC_SYNCPOINT_NOT_AVAILABLE)
    PRINTREAS(MQRC_TRIGGER_CONTROL_ERROR)
    PRINTREAS(MQRC_TRIGGER_DEPTH_ERROR)
    PRINTREAS(MQRC_TRIGGER_MSG_PRIORITY_ERR)
    PRINTREAS(MQRC_TRIGGER_TYPE_ERROR)
    PRINTREAS(MQRC_TRUNCATED_MSG_ACCEPTED)
    PRINTREAS(MQRC_TRUNCATED_MSG_FAILED)
    PRINTREAS(MQRC_UNKNOWN_ALIAS_BASE_Q)
    PRINTREAS(MQRC_UNKNOWN_OBJECT_NAME)
    PRINTREAS(MQRC_UNKNOWN_OBJECT_Q_MGR)
    PRINTREAS(MQRC_UNKNOWN_REMOTE_Q_MGR)
    PRINTREAS(MQRC_WAIT_INTERVAL_ERROR)
    PRINTREAS(MQRC_XMIT_Q_TYPE_ERROR)
    PRINTREAS(MQRC_XMIT_Q_USAGE_ERROR)
    PRINTREAS(MQRC_NOT_OPEN_FOR_PASS_ALL)
    PRINTREAS(MQRC_NOT_OPEN_FOR_PASS_IDENT)
    PRINTREAS(MQRC_NOT_OPEN_FOR_SET_ALL)
    PRINTREAS(MQRC_NOT_OPEN_FOR_SET_IDENT)
    PRINTREAS(MQRC_CONTEXT_HANDLE_ERROR)
    PRINTREAS(MQRC_CONTEXT_NOT_AVAILABLE)
    PRINTREAS(MQRC_SIGNAL1_ERROR)
    PRINTREAS(MQRC_OBJECT_ALREADY_EXISTS)
    PRINTREAS(MQRC_OBJECT_DAMAGED)
    PRINTREAS(MQRC_RESOURCE_PROBLEM)
    PRINTREAS(MQRC_ANOTHER_Q_MGR_CONNECTED)
    PRINTREAS(MQRC_UNKNOWN_REPORT_OPTION)
    PRINTREAS(MQRC_SUPPRESSED_BY_EXIT)
    PRINTREAS(MQRC_FORMAT_ERROR)
    PRINTREAS(MQRC_SOURCE_CCSID_ERROR)
    PRINTREAS(MQRC_SOURCE_INTEGER_ENC_ERROR)
    PRINTREAS(MQRC_SOURCE_DECIMAL_ENC_ERROR)
    PRINTREAS(MQRC_SOURCE_FLOAT_ENC_ERROR)
    PRINTREAS(MQRC_TARGET_CCSID_ERROR)
    PRINTREAS(MQRC_TARGET_INTEGER_ENC_ERROR)
    PRINTREAS(MQRC_TARGET_DECIMAL_ENC_ERROR)
    PRINTREAS(MQRC_TARGET_FLOAT_ENC_ERROR)
    PRINTREAS(MQRC_NOT_CONVERTED)
    PRINTREAS(MQRC_CONVERTED_MSG_TOO_BIG)
    PRINTREAS(MQRC_ADAPTER_STORAGE_SHORTAGE)
    PRINTREAS(MQRC_ADAPTER_CONN_LOAD_ERROR)
    PRINTREAS(MQRC_ADAPTER_SERV_LOAD_ERROR)
    PRINTREAS(MQRC_ADAPTER_DEFS_ERROR)
    PRINTREAS(MQRC_ADAPTER_DEFS_LOAD_ERROR)
    PRINTREAS(MQRC_ADAPTER_DISC_LOAD_ERROR)
    PRINTREAS(MQRC_CICS_WAIT_FAILED)
    PRINTREAS(MQRC_SOURCE_LENGTH_ERROR)
    PRINTREAS(MQRC_TARGET_LENGTH_ERROR)
    PRINTREAS(MQRC_SOURCE_BUFFER_ERROR)
    PRINTREAS(MQRC_TARGET_BUFFER_ERROR)
    PRINTREAS(MQRC_DBCS_ERROR)
    PRINTREAS(MQRC_TRUNCATED)
    PRINTREAS(MQRC_ASID_MISMATCH)
    PRINTREAS(MQRC_CONN_ID_IN_USE)
    PRINTREAS(MQRC_Q_MGR_QUIESCING)
    PRINTREAS(MQRC_Q_MGR_STOPPING)
    PRINTREAS(MQRC_DUPLICATE_RECOV_COORD)
    PRINTREAS(MQRC_PMO_ERROR)
    PRINTREAS(MQRC_API_EXIT_NOT_FOUND)
    PRINTREAS(MQRC_API_EXIT_LOAD_ERROR)
    PRINTREAS(MQRC_REMOTE_Q_NAME_ERROR)
    PRINTREAS(MQRC_GMO_ERROR)
    PRINTREAS(MQRC_PAGESET_FULL)
    PRINTREAS(MQRC_PAGESET_ERROR)
    PRINTREAS(MQRC_NAME_NOT_VALID_FOR_TYPE)
    PRINTREAS(MQRC_UNEXPECTED_ERROR)
    PRINTREAS(MQRC_UNKNOWN_XMIT_Q)
    PRINTREAS(MQRC_UNKNOWN_DEF_XMIT_Q)
    PRINTREAS(MQRC_DEF_XMIT_Q_TYPE_ERROR)
    PRINTREAS(MQRC_DEF_XMIT_Q_USAGE_ERROR)
    PRINTREAS(MQRC_NAME_IN_USE)
    PRINTREAS(MQRC_CONNECTION_QUIESCING)
    PRINTREAS(MQRC_CONNECTION_STOPPING)
    PRINTREAS(MQRC_ADAPTER_NOT_AVAILABLE)
    PRINTREAS(MQRC_MSG_ID_ERROR)
    PRINTREAS(MQRC_CORREL_ID_ERROR)
    PRINTREAS(MQRC_FILE_SYSTEM_ERROR)
    PRINTREAS(MQRC_NO_MSG_LOCKED)
    PRINTREAS(MQRC_CONNECTION_NOT_AUTHORIZED)
    PRINTREAS(MQRC_MSG_TOO_BIG_FOR_CHANNEL)
    PRINTREAS(MQRC_CALL_IN_PROGRESS)
    PRINTREAS(MQRC_Q_MGR_ACTIVE)
    PRINTREAS(MQRC_Q_MGR_NOT_ACTIVE)
    PRINTREAS(MQRC_Q_DEPTH_HIGH)
    PRINTREAS(MQRC_Q_DEPTH_LOW)
    PRINTREAS(MQRC_Q_SERVICE_INTERVAL_HIGH)
    PRINTREAS(MQRC_Q_SERVICE_INTERVAL_OK)
    PRINTREAS(MQRC_HCONFIG_ERROR)
    PRINTREAS(MQRC_FUNCTION_ERROR)
    PRINTREAS(MQRC_CHANNEL_STARTED)
    PRINTREAS(MQRC_CHANNEL_STOPPED)
    PRINTREAS(MQRC_CHANNEL_CONV_ERROR)
    PRINTREAS(MQRC_SERVICE_NOT_AVAILABLE)
    PRINTREAS(MQRC_INITIALIZATION_FAILED)
    PRINTREAS(MQRC_TERMINATION_FAILED)
    PRINTREAS(MQRC_UNKNOWN_Q_NAME)
    PRINTREAS(MQRC_SERVICE_ERROR)
    PRINTREAS(MQRC_Q_ALREADY_EXISTS)
    PRINTREAS(MQRC_USER_ID_NOT_AVAILABLE)
    PRINTREAS(MQRC_UNKNOWN_ENTITY)
    PRINTREAS(MQRC_UNKNOWN_AUTH_ENTITY)
    PRINTREAS(MQRC_UNKNOWN_REF_OBJECT)
    default:
      printf("It's an unknown reason, %ld\n",
              reason);
      break;
  }
}
void rmvexpired(char * thequeue)
{
  /*******************************************************************/
  /* declare LOCAL variables                                         */
  /*******************************************************************/
  MQOD     od = {MQOD_DEFAULT};       /* Object Descriptor           */
  MQLONG   OpenCode;                  /* MQOPEN completion code      */
  MQHOBJ   Hobj;                      /* object handle               */
  MQLONG   CompCode;                  /* completion code             */
  MQLONG   Reason;                    /* reason code                 */
  MQGMO    gmo = {MQGMO_DEFAULT};     /* get message options         */
  MQMD     md = {MQMD_DEFAULT};       /* Message Descriptor          */
  MQLONG   outmsglen;                 /* message length received     */
  MQLONG   C_options;                 /* MQCLOSE options             */
  MQLONG   msgcount = 0;              /* msg counter                 */

  /*******************************************************************/
  /* Initialize object descriptor for subject queue                  */
  /*******************************************************************/
  strcpy(od.ObjectName, thequeue);

  /*******************************************************************/
  /* Open the queue for browse only. Use of the queue is controlled  */
  /* by the queue definition here                                    */
  /*******************************************************************/
  MQOPEN(Hcon,                        /* connection handle           */
         &od,                         /* object descriptor for queue */
         MQOO_FAIL_IF_QUIESCING
           + MQOO_BROWSE,             /* open options                */
         &Hobj,                       /* object handle               */
         &CompCode,                   /* completion code             */
         &Reason);                    /* reason code                 */

  /*******************************************************************/
  /* report reason, if any                                           */
  /*******************************************************************/
  if (Reason != MQRC_NONE)
  {
    printf("MQOPEN (%s) ==> %ld\n", od.ObjectName, Reason);
    printreas(Reason);
  }

  OpenCode = CompCode;                 /* keep for conditional close */

  /*******************************************************************/
  /* go through the whole queue browsing the messages                */
  /*******************************************************************/
  while (CompCode != MQCC_FAILED)
  {
    /*****************************************************************/
    /* We don't really need to get the message,just to touch it to   */
    /* make the expired flag take effect so don't need a big buffer  */
    /*****************************************************************/
    gmo.Options = MQGMO_NO_WAIT + MQGMO_BROWSE_NEXT
                                + MQGMO_ACCEPT_TRUNCATED_MSG;

    /*****************************************************************/
    /* clear selectors to get messages in sequence                   */
    /*****************************************************************/
    memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
    memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));

    /*****************************************************************/
    /* browse the message                                            */
    /*****************************************************************/
    MQGET(Hcon,                        /* connection handle          */
          Hobj,                        /* object handle              */
          &md,                         /* message descriptor         */
          &gmo,                        /* get message options        */
          0,                           /* buffer length              */
          outmsg,                      /* outmsg message buffer      */
          &outmsglen,                  /* message length             */
          &CompCode,                   /* completion code            */
          &Reason);                    /* reason code                */

    /*****************************************************************/
    /* report reason, if any                                         */
    /*****************************************************************/
    if (Reason != MQRC_NONE && Reason != MQRC_TRUNCATED_MSG_ACCEPTED)
    {
      if (Reason != MQRC_NO_MSG_AVAILABLE)
      {
        printf("MQGET ==> %ld\n", Reason);
        printreas(Reason);
      }
    }
    else
    {
      msgcount++;
    }
  }                                    /* end msg processing loop    */
  if (Reason == MQRC_NO_MSG_AVAILABLE)
  {
    printf("%ld msgs on queue\n",msgcount);
  }

  /*******************************************************************/
  /* close the queue - if it was opened                              */
  /*******************************************************************/
  if (OpenCode != MQCC_FAILED)
  {
    C_options = 0;                   /* no close options             */
    MQCLOSE(Hcon,                    /* connection handle            */
           &Hobj,                    /* object handle                */
           C_options,
           &CompCode,                /* completion code              */
           &Reason);                 /* reason code                  */
    /*****************************************************************/
    /* report reason, if any                                         */
    /*****************************************************************/
    if (Reason != MQRC_NONE)
    {
      printf("MQCLOSE ==> %ld\n", Reason);
      printreas(Reason);
    }
  }
}
